@@ -451,7 +451,7 @@ class TestSingleStore(Validator):
             "SELECT REGEXP_MATCH('adog', 'O', 'c')",
             read={
                 # group, position, occurrence parameters are not supported in SingleStore, so they are ignored
-                "": "SELECT REGEXP_EXTRACT_ALL('adog', 'O', 1, 1, 'c', 'gr1')",
+                "": "SELECT REGEXP_EXTRACT_ALL('adog', 'O', 1, 'c', 1, 'gr1')",
                 "singlestore": "SELECT REGEXP_MATCH('adog', 'O', 'c')",
             },
         )
@@ -965,3 +965,6 @@ class TestSingleStore(Validator):
         self.validate_identity(
             "CREATE TABLE ComputedColumnConstraint (points INT, score AS (points * 2) PERSISTED BIGINT NOT NULL)"
         )
+
+    def test_dcolonqmark(self):
+        self.validate_identity("SELECT * FROM employee WHERE JSON_MATCH_ANY(payroll::?names)")
