@@ -51,11 +51,11 @@ x << 1
 x >> 1
 x >> 1 | 1 & 1 ^ 1
 x || y
-x[ : ]
-x[1 : ]
-x[ : 2]
-x[1 : 2]
-x[-4 : -1]
+x[:]
+x[1:]
+x[:2]
+x[1:2]
+x[-4:-1]
 1 - -1
 - -5
 dec.x + y
@@ -471,8 +471,6 @@ WITH a AS (SELECT 1) SELECT 1 UNION SELECT 2
 WITH a AS (SELECT 1) SELECT 1 INTERSECT SELECT 2
 WITH a AS (SELECT 1) SELECT 1 EXCEPT SELECT 2
 WITH a AS (SELECT 1) SELECT 1 EXCEPT SELECT 2
-WITH sub_query AS (SELECT a FROM table) (SELECT a FROM sub_query)
-WITH sub_query AS (SELECT a FROM table) ((((SELECT a FROM sub_query))))
 (SELECT 1) UNION (SELECT 2)
 (SELECT 1) UNION SELECT 2
 SELECT 1 UNION (SELECT 2)
@@ -948,3 +946,6 @@ WITH UNNEST AS (SELECT 1 AS UNNEST) SELECT UNNEST FROM UNNEST
 SELECT * FROM tbl GROUP BY GROUPING SETS ((a + 1, b * 1), c, CUBE (a, b), ROLLUP (c, d), (a + y, b * 1), ())
 SELECT * FROM tbl GROUP BY GROUPING SETS (GROUPING SETS (course), GROUPING SETS (type), CUBE (a), ROLLUP (b))
 SELECT analyze FROM (SELECT 1 AS analyze)
+SELECT 'Ac' ILIKE 'a%c' ESCAPE NULL
+SELECT CURRENT_DATABASE()
+SELECT CURRENT_SCHEMAS(arg_bool)
