@@ -25,8 +25,22 @@ bool;
 false;
 bool;
 
+x is null;
+bool;
+
+x is not null;
+bool;
+
 null;
+UNKNOWN;
+
+# dialect: spark
 null;
+NULL;
+
+# dialect: databricks
+null;
+NULL;
 
 null and false;
 bool;
@@ -92,3 +106,35 @@ DECIMAL(18,2);
 
 CASE WHEN x THEN NULL ELSE CAST(y AS DECIMAL(18, 2)) END;
 DECIMAL(18,2);
+
+# dialect: bigquery
+CASE WHEN TRUE THEN '2010-01-01' ELSE DATE '2020-02-02' END;
+DATE;
+
+# dialect: bigquery
+CASE WHEN TRUE THEN '2010-01-01' WHEN FALSE THEN DATE '2020-02-02' ELSE '1990-01-01' END;
+DATE;
+
+# dialect: bigquery
+CASE WHEN TRUE THEN DATETIME '2020-02-02 00:00:00' ELSE '2010-01-01' END;
+DATETIME;
+
+# dialect: bigquery
+CASE WHEN TRUE THEN TIMESTAMP '2020-02-02 00:00:00' ELSE '2010-01-01' END;
+TIMESTAMP;
+
+# dialect: bigquery
+NULL;
+INT64;
+
+# dialect: bigquery
+ARRAY(SELECT 'foo' UNION ALL SELECT 'bar');
+ARRAY<STRING>;
+
+# dialect: bigquery
+ARRAY(SELECT 1 UNION ALL SELECT 2 UNION ALL SELECT 3);
+ARRAY<INT64>;
+
+# dialect: bigquery
+ARRAY(SELECT 1 UNION ALL SELECT 2.5);
+ARRAY<FLOAT64>;
